#ifndef SCOPE_H
#define SCOPE_H

/* LX200 protocol...
see lx200.txt for lx200 protocol notes;
the serial interrupt routine reads input from an external device outputting
lx200 protocol serial commands;
every bios clock tick serial new characters stored in the serial input buffer
are copied to the lx200 circular queue;
these characters are then scanned and acted upon: several commands may be acted
upon in a single bios clock tick;
setting the test mode to lx200 will display inputted characters and the
deciphered commands */

#define LX200_ACK 6
#define DegSym 223
#define LX200_OK '1'
/* must be 2^n in size */
#define LX200QueueSize 64
/* must be 2^n - 1 in size */
#define LX200QueueEnd 63
/* must be 2^n in size */
#define LX200_Cmd_Array_Size 8
/* must be 2^n - 1 in size */
#define LX200_Cmd_Array_End 7
#define LX200_CMD_STR_SIZE 3

typedef enum
{
	NoAck,
	Ack,
	GetRa,
	GetDec,
	GetAlt,
	GetAz,
	GetSidT,
	GetLat,
	GetLong,
	MoveDirRateNorth,
	MoveDirRateSouth,
	MoveDirRateEast,
	MoveDirRateWest,
	StartSlew,
	StopSlew,
	StopMotionNorth,
	StopMotionSouth,
	StopMotionEast,
	StopMotionWest,
	SetMotionRateGuide,
	SetMotionRateCenter,
	SetMotionRateFind,
	SetMotionRateSlew,
	SetRa,
	SetDec,
	SetSidT,
	SetLocalT,
	SetDate,
	Sync,
	ToggleLongFormat,
	LX200_Unfinished_Cmd,
	LX200_Unknown_Cmd0,
	LX200_Unknown_Cmd1,
	LX200_Ignored_Cmd,
	MaxLX200_COMMAND_TYPE
}LX200_COMMAND_TYPE;

typedef struct
{
	LX200_COMMAND_TYPE LX200_Cmd_Type;
	/* beginning index into lx200 queue of characters belonging to this command */
	int BegIx;
} LX200_CMD_ARRAY;

struct Position In;

/* toggles between standard and long format for coordinates */
Flag LX200_LongFormat;
/* 2 character descriptions of command names */
char LX200_Cmd_Str[MaxLX200_COMMAND_TYPE][LX200_CMD_STR_SIZE];
LX200_COMMAND_TYPE LX200_Motor_Cmd, LX200_Hold_Motor_Cmd, LX200_Speed_Cmd;
int LX200_Cmd_Array_Ix;
/* array of commands */
LX200_CMD_ARRAY LX200_Cmd_Array[LX200_Cmd_Array_Size];
/* circular queue that LX200 protocol serial input is read into */
char LX200Queue[LX200QueueSize];
int LX200_Ix, Beg_LX200_Ix;
/* count of commands processed */
int LX200_Cmd_Count;
struct AZLong LX200SlewSteps;

const enum
{
	MenuQuit,
	MenuDataFile,
	MenuScrollTour,
	MenuDosShell,
	MenuGuide,
	MenuHotkeys,

	MenuHandPad,
	MenuTrack,
	MenuMsSpeed,
	MenuGuideSpeed,
	MenuReadSlew,
	MenuWriteSlew,
	MenuLX200Display,
	MenuChangeHsParms,
	MenuChangeMsParms,

	MenuPECOnOff,
	MenuPECReload,
	MenuPECDisplay,
	MenuPECAverageFiles,
	MenuPECGuidingDisplay,
	MenuPECZeroOut,

	MenuInit1,
	MenuInit2,
	MenuInit3,
	MenuAltOffset,
	MenuDisplayInit,
	MenuOptimizeInit,
	MenuInitEncoders,

	MenuInputEquat,
	MenuOffsetEquat,
	MenuDriftEquat,
	MenuInputAltaz,
	MenuOffsetAltaz,
	MenuDriftAltaz,

	MenuMoveEquat,
	MenuMoveAltaz,
	MenuMoveHome,
	MenuSav1,
	MenuRes1,
	MenuSav2,
	MenuRes2,

	MenuResetEquat,
	MenuResetAltaz,
	MenuResetHome,
	MenuResetToEncoders,
	MenuGEMMeridianFlip,
	MenuFRReset,
	MaxMenu
};

void InitLX200Input( void);
void InsertLX200CmdIntoArray( const LX200_COMMAND_TYPE LX200_Cmd_To_Add,
const int LX200_Ix);
void ReadLX200Input( void);
void LX200_Write_Ra( void);
void LX200_Write_Dec( void);
void LX200_Write_Alt( void);
void LX200_Write_Az( void);
void LX200_Write_SidT( void);
void LX200_Write_Lat( void);
void LX200_Write_Long( void);
void LX200_Read_Ra( const int Ix);
void LX200_Read_Dec( const int Ix);
void LX200_Read_SidT( const int Ix);
void LX200_Read_LocalT( const int Ix);
void LX200_Read_CurrentDate( const int Ix);
void ProcessLX200Cmds( void);
void ProcessLX200_Motor_Cmd( void);

/* handpad event... */

#define LastDirSize 6

/* field rotation motor uses first two lines (pins 1, 14 of the parallel port)
of the 4 bits in-out port (parallel port base + 2) */

#define FRPort PPortAddrInOutNibble
#define FRPulseBit 1
#define FRDirBit 2

#define RecordEquatFile "RECORD.DAT"

typedef enum
{
	HandpadOff,
	InitAutoOn,
	Init1On,
	Init2On,
	Init3On,
	GuideOn,
	GuideStayOn,
	GuideStaySaveOn,
	GuideDragOn,
	GrandTour,
	ScrollTour,
	RecordOn,
	ToggleTrack,
	HandpadFR,
	MaxHandpadFlag
}HANDPAD_FLAG;

HANDPAD_FLAG HandpadFlag;
/* tracking to current Ra, Dec stopped when flag == false; */
/* altaz mvmt initiated by handpad allowed regardless of flag */
Flag TrackFlag;
/* non-zero backlash in either Alt or Az */
Flag BacklashFlag;
/* check for motor direction oscillation caused by backlash updating */
Flag CheckBacklashOscillation;
/* use PEC */
Flag PECFlag;
/* grand tour on */
Flag GrandTourFlag;
/* scroll tour on */
Flag ScrollFlag;

/* variables relating to field rotation... */

/* if 1, go ahead and do field rotation motor code */
Flag DoFR;
/* field rotation motor step size caused by a single pulse */
double FRStepSize;
/* current field rotation angle of motor/focuser assembly */
double FRMotorAngle;
/* reset FRMotorAngle if difference between field rotation angle and current
field rotation angle of motor/focuser is greater than this */
double ResetFRMotorAngleThreshold;
/* direction of field rotation motor movement: set either to FRDirBit (CW) or
Off (CCW) */
int FRDir;
Byte FRPulse;
Flag HandpadFRMotorControlOnFlag;
int FRDiagPulseHigh;
int FRDiagPulseLow;
int FRDiagReset;

/* counts of consecutive slews when scope should be microstepping tracking...
if counts reach max values defined in the config file, then scope resets current
equatorial to current altazimuth */
int ConsecutiveSlews;

/* move initiated by MoveToCurrentRaDec() */
Flag MoveToCurrentRaDecFlag;
/* if true, add in drift */
Flag DriftEquatFlag;
Flag DriftAltazFlag;
/* in radians */
struct AZDouble Backlash;
/* direction that backlash has been taken up, or reduced to zero, in */
struct AZFlag BacklashDir;
/* steps to move to take up backlash */
struct AZLong BacklashMs;
struct AZDouble HoldMsTick;
/* max # of microsteps per clock tick */
double MaxMsTick;
struct AZDouble Delta;
double StartDriftT, EndDriftT;
struct AZDouble AccumDrift;
struct AZDouble HoldAZ;
/* equat drift per hour, altaz drift per min */
struct Position Drift, Offset, DriftTick;
/* drift to drag guide star across knife edge autoguider, values from
config.dat */
struct Position Drag, DragTick;

void InitHPEvent( void);
void SetAccumMsToCurrentAltaz( void);
void SetCurrentAltazToAccumMs( void);
void HPEventMoveHs( void);
void HPEventMoveMs( void);
void AltazMoveMs( void);
Flag MoveBacklash( void);
void FRMotorOff( void);
void MoveFR( void);
void CheckSiderostatAltaz( void);
void MoveToCurrentRaDec( void);
Flag SetDirDistanceStepsThenMove( void);
void HPEventGetEquat( void);
void HPEventInitMatrix( const int Init);
void UpdateDriftCalculatedFromGuide( void);
void RecordEquat( void);
Flag IACA_Event( void);
int FindClosestInit( void);
void SetEndDriftT( void);
void ZeroDrift( void);
void AddDragToDrift( void);
void RemoveDragFromDrift( void);
void CalcCommonDriftVars( void);
void CheckMiscEvents( void);
void ProcessHPEvents( void);
void Track( void);

/* keyboard event... */

#define MaxNameArray 24
#define FilenameSize 13
#define MaxCoord 24
#define MaxMainStr 14
#define EncoderOffsetFile "ENCOFF.DAT"
#define GuideStartupMarFile "C:\\GUIDE\\STARTUP.MAR"
#define ScopeStartupMarFile "C:\\GUIDE\\SCOPE.MAR"
#define SlewFile "SLEW.DAT"
#define SlewOutFile "SLEW_OUT.DAT"
#define OutGuideFile "OUTGUIDE.DAT"
#define NotAvailableStr "N/A    "
#define DataFile "*.dat"
#define ScrollFile "*.scr"
/* # of menu categories to display from left to right in a single row */
#define MaxMenuCats 7
/* # of menu subtitles to display in a single row */
#define MaxMenuSubs 5
/* MaxMenuSubs is # of display columns of menu subtitles, so make total # of
displayed subtitles this number times the max number of display rows */
#define MaxSubTitles (MaxMenuSubs * 3)

/* each main menu category has a number of associated menu subtitles;
when a main menu category is selected, its menu subtitles are displayed */

struct SubTitle
{
	char Title[NameSize];
	char Desc[MaxX];
	int MenuItem;
};

/* main menu categories displayed in a row from left to right */
struct MenuCat
{
	char Title[NameSize];
	int NumSubTitles;
	struct SubTitle SubTitles[MaxSubTitles];
};

struct MenuCat MenuArray[MaxMenuCats] =
{
	{"File", 6,
	{
		{"quit", "quit the program", MenuQuit},
		{"data file", "load a data file containing object coordinates", MenuDataFile},
		{"scroll tour", "load a file of slow smooth scrolling motions", MenuScrollTour},
		{"DOS shell", "shell to DOS", MenuDosShell},
		{"guide.bat", "call DOS version of Project Pluto Guide (cd-rom)", MenuGuide},
		{"hotkeys", "display a listing of hotkeys", MenuHotkeys},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"Control", 9,
	{
		{"handpad mode", "cycle through hand paddle mode settings", MenuHandPad},
		{"track on/off", "toggle tracking on and off", MenuTrack},
		{"Ms speed", "set the microstepping speed", MenuMsSpeed},
		{"guide speed", "set the speed to use during guide mode", MenuGuideSpeed},
		{"read slew", "read slew file generated by external control program", MenuReadSlew},
		{"write slew", "write slew file for use by external control program", MenuWriteSlew},
		{"LX200 display", "display LX200 protocol commands and character buffer", MenuLX200Display},
		{"Hs parms", "change halfstepping parameters", MenuChangeHsParms},
		{"Ms parms", "change microstepping parameters", MenuChangeMsParms},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"PEC", 6,
	{
		{"on/off", "toggle periodic error corection on and off", MenuPECOnOff},
		{"reload", "reload PEC from PECFile", MenuPECReload},
		{"display", "display working PEC in memory", MenuPECDisplay},
		{"avg files", "average the PEC data files created from guiding efforts", MenuPECAverageFiles},
		{"guiding", "extract PEC data from guiding efforts", MenuPECGuidingDisplay},
		{"zero-out",  "zero-out working PEC data in memory", MenuPECZeroOut},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"Init", 7,
	{
		{"init 1", "set initialization position #1", MenuInit1},
		{"init 2", "set initialization position #2", MenuInit2},
		{"init 3", "set initialization position #3", MenuInit3},
		{"alt offset", "add in altitude offset and re-initialize", MenuAltOffset},
		{"display", "display the initialization settings", MenuDisplayInit},
		{"optimize", "select closest initialization positions", MenuOptimizeInit},
		{"encoders", "re-initialize the encoder interface box", MenuInitEncoders},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"Coord", 10,
	{
		{"input equat", "enter new input equatorial coordinates", MenuInputEquat},
		{"offset equat", "enter offset from current equatorial coordinates", MenuOffsetEquat},
		{"drift equat", "enter equatorial drift coordinates", MenuDriftEquat},
		{"input altaz", "enter new input altazimuth coordinates", MenuInputAltaz},
		{"offset altaz", "enter offset from current altazimuth coordinates", MenuOffsetAltaz},
		{"drift altaz", "enter altazimuth drift coordinates", MenuDriftAltaz},
		{"sav 1", "saves current equatorial coordinates into #1 buffer", MenuSav1},
		{"res 1", "restore saved #1 equatorial coordinates into input fields", MenuRes1},
		{"sav 2", "saves current equatorial coordinates into #2 buffer", MenuSav2},
		{"res 2", "restore saved #2 equatorial coordinates into input fields", MenuRes2},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"Move", 3,
	{
		{"equat", "move to input equatorial coordinates", MenuMoveEquat},
		{"altaz", "move to input altazimuth coordinates", MenuMoveAltaz},
		{"home", "move to home coordinates", MenuMoveHome},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
	{"Reset", 6,
	{
		{"equat", "reset to input equatorial coordinates", MenuResetEquat},
		{"altaz", "reset to input altazimuth coordinates", MenuResetAltaz},
		{"home", "reset to home coordinates", MenuResetHome},
		{"encoders", "reset to encoder coordinates", MenuResetToEncoders},
		{"GEM flip", "reset coordinates in preparation for GEM meridian flip", MenuGEMMeridianFlip},
		{"field rot", "reset field rotation motor to current field rotation angle", MenuFRReset},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
		{"", "", None},
	},
	},
};

/* starting X values for display of menu categories */
int MenuCatX[MaxMenuCats];
/* starting X values for display of menu subtitles; if number of subtitles
exceed number of display columns, then subtitles displayed in next row,
repeating the X values for the columns */
int MenuSubX[MaxMenuSubs];

struct Frame
{
	int Left;
	int Top;
	int Right;
	int Bottom;
	char* Title;
};

struct FilenameArray
{
	char Name[FilenameSize];
};

struct Object
{
	char Name[ObjectNameSize];
	struct HMSH RaHMSH;
	struct DMS DecDMS;
};

typedef enum
{
	equat_abs,
	equat_off,
	no_type
}COORD_TYPE;

struct SCROLL
{
	COORD_TYPE CoordType;
	double Ra;
	struct HMSH RaHMSH;
	double Dec;
	struct DMS DecDMS;
	double Sec;
	char Note[NameSize];
	struct SCROLL* PrevScroll;
	struct SCROLL* NextScroll;
};

/* MenuColor and DefaultBackground */
int MenuText;
/* SelectColor and SelectBackground */
int SelectText;
/* CurrentColor and DefaultBackground */
int CurrentText;
/* DisplayColor and DefaultBackground */
int DisplayText;
/* DefaultColor and DefaultBackground */
int DefaultText;
/* BorderColor and DefaultBackground */
int BorderText;
/* TitleColor and DefaultBackground */
int TitleText;
/* currently selected menu category and subtitle in that category */
int CurrentMenuCat;
int CurrentMenuSub;
/* newly selected menu category and subtitle in that category */
int NewMenuCat;
int NewMenuSub;
/* if move exceeds this, then confirm move */
long MoveHsMsgSteps;
/* encoder Alt, Az; encoder Alt, Az offset (Current - encoder = offset) */
struct AZDouble EncoderAZ, EncoderOffset;
double EncoderErrorThreshold;
double TrackEncoderErrorThreshold;
Flag LX200DisplayFlag;
Flag ValidResponseFlag;
Flag GrandTourLoaded;
Flag ScrollLoaded;
Flag CurrentScrollFlag;
Flag ScrollUnderway;
Flag MsParmsAllowedFlag;
Flag MsParmsTitlesWritten;
Flag QuitFlag;
int SpawnReturn;
int GrandTourRecNum;
char FullyQualifiedSlewFile[NameSize];
char FullyQualifiedSlewOutFile[NameSize];
char CurDir[MAXPATH];
struct Position SavedIn, SavedIn2;
struct SCROLL* FirstScroll;
struct SCROLL* LastScroll;
struct SCROLL* CurrentScroll;
struct SCROLL* PrevScroll;
struct SCROLL* NextScroll;
char Filename[NameSize] = "bstars.dat";
char GrandTourFilename[NameSize];
char ScrollFilename[NameSize];
FILE* OutGuideFilePtr;

Flag Threshold;

char* SlewText[] =
{
	"RampU",
	"MaxU ",
	"MaxD ",
	"RampD",
	"Done "
};

char* AbortText[] =
{
	"            ",
	"Pad-Abort   ",
	"KeyB-Abort  ",
	"Alt-Lo-Abort",
	"Alt-Hi-Abort",
	"Az-Lo-Abort ",
	"Az-Hi-Abort ",
};

/* to save screen window */
char WinBuffer[MaxX*MaxY*2];
const struct Frame MsgFrame = {12, 9, 68, 17, " Message "};
const struct Frame CoordFrame = {4, 5, 76, 18, " Coordinates "};
const struct Frame DataFileFrame = {10, 5, 70, 20, " Data Files "};
const struct Frame InitFrame = {10, 10, 70, 22, " Initialization "};
const struct Frame HotkeyFrame = {4, 1, 76, 25, " Hotkeys "};
const struct Frame MainFrame = {1, 1, MaxX, MaxY,
" Scope Control Program Compiled June-23-1999 by Mel Bartels "};

/*
			1         2         3         4         5         6         7         8
12345678901234567890123456789012345678901234567890123456789012345678901234567890
					 Scope Control Compiled mm-dd-yy by Mel Bartels                  00
																										  01
																										  02
																										  03
																										  04
																										  05
																										  06
																										  07
																										  08
		 Current     Input      Drift        Encoders        Refract  FieldRot    09
Alt:   +dd.ddd     +dd.ddd    +dd.ddd/m    +dd.ddd/999999  +dd.ddd  ddd.ddd     10
Az:    ddd.ddd     ddd.ddd    ddd.ddd/m    ddd.ddd/999999           ddd.ddd     11
																										  12
Ra:    hh:mm:ss    hh:mm:ss   hh:mm:ss/h                                        13
Dec:   +dd:mm.ss   +dd:mm.ss  +dd:mm.ss/h                                       14
												  Init							  PWM	   		  15
File                                  Handpad                                   16
Object                                Track                                     17
Sidereal Time                         PEC                                       18
Date/Time                             Microstep     /sec  Guide       /sec      29
LX200 										                                            20
																										  21
Slew                                                                            22
Ms																										  23
																										  24
12345678901234567890123456789012345678901234567890123456789012345678901234567890
			1         2         3         4         5         6         7         8
*/

char* MainStr_9 =
"       Current     Input      Drift        Encoders        Refract  FieldRot";
char* MainStr_10 =
"Alt:                                 \/m";
char* MainStr_11 =
"Az:                                  \/m";
char* MainStr_13 =
"Ra:                                   \/h";
char* MainStr_14 =
"Dec:                                   \/h";
char* MainStr_15 =
"                                      Init                      PWM";
char* MainStr_16 =
"File                                  Handpad";
char* MainStr_17 =
"Object                                Track";
char* MainStr_18 =
"Sidereal Time                         PEC";
char* MainStr_19 =
"Date/Time                             Microstep     \"/sec  Guide     \"/sec";
char* MainStr_20 =
"LX200";
char* MainStr_22 =
"Slew";
char* MainStr_23 =
"Ms";

/* following section defines X, Y coordinates... */

/* reused X coordinates */
const enum
{
	MenuStartX = 2, MenuEndX = 79,
	MenuDescX = 6,
	MenuCurrentX = 9, MenuInputX = 21, MenuDriftX = 32, MenuEncoderX = 45,
	MenuEncoderCountX = 53, MenuRefractX = 61, MenuFRX = 70,
	MenuInitX = 49, MenuPWMX = 70,
	MenuFileX = 9, MenuHandpadX = 49, MenuButtonsX = 69,
	MenuObjectX = 11, MenuTrackX = 49,
	MenuSidTX = 18, MenuPECX = 49,
	MenuDateTimeX = 12, MenuMsX = 50, MenuGuideX = 67,
	MenuLX200X = 8,
	MenuSlewX = 8,
	MenuMsParmsX = 5
};

/* reused Y coordinates; top line = 0 */
const enum
{
	MenuCatsY = 1,
	MenuSubLine1Y = 2,
	MenuSubMenuY = 3,
	MenuDescY = 7,
	MenuSubLine2Y = 8,
	MenuCoordTitleY = 9,
	MenuAltY = 10,
	MenuAzY = 11,
	MenuRaY = 13,
	MenuDecY = 14,
	MenuInitY = 15, MenuPWMY = 15,
	MenuFileY = 16, MenuHandpadY = 16, MenuButtonsY = 16,
	MenuObjectY = 17, MenuTrackY = 17,
	MenuSidTY = 18, MenuPECY = 18,
	MenuDateTimeY = 19, MenuMsY = 19, MenuGuideY = 19,
	MenuLX200Y = 20,
	MenuSlewY = 22,
	MenuMsParmsY = 23
};

/* named indexes into DisplayXY[] */
const enum
{
	DisplayCurrentAlt, DisplayInputAlt, DisplayDriftAlt, DisplayEncoderAlt,
	DisplayEncoderCountAlt, DisplayRefract, DisplayFR,

	DisplayCurrentAz, DisplayInputAz, DisplayDriftAz, DisplayEncoderAz,
	DisplayEncoderCountAz,

	DisplayCurrentRa, DisplayInputRa, DisplayDriftRa,
	DisplayCurrentDec, DisplayInputDec, DisplayDriftDec,
	DisplayInitStatus, DisplayPWMReps,
	DisplayFile, DisplayHandpad, DisplayButtons,
	DisplayObject, DisplayTrack,
	DisplaySidT, DisplayPEC,
	DisplayDateTime, DisplayMsSpeed, DisplayGuideSpeed,
	DisplayLX200,
	DisplayHsSlew, DisplayHsAbort, DisplayHsSteps,
	DisplayMsParmsAllowed, DisplayMsPWMs,
	MaxDisplay
};

const struct XY DisplayXY[MaxDisplay] =
{
	{MenuCurrentX, MenuAltY}, {MenuInputX, MenuAltY}, {MenuDriftX, MenuAltY},
	{MenuEncoderX, MenuAltY}, {MenuEncoderCountX, MenuAltY},
	{MenuRefractX, MenuAltY}, {MenuFRX, MenuAltY},

	{MenuCurrentX, MenuAzY}, {MenuInputX, MenuAzY}, {MenuDriftX, MenuAzY},
	{MenuEncoderX, MenuAzY}, {MenuEncoderCountX, MenuAzY},

	{MenuCurrentX, MenuRaY}, {MenuInputX, MenuRaY}, {MenuDriftX, MenuRaY},
	{MenuCurrentX, MenuDecY}, {MenuInputX, MenuDecY}, {MenuDriftX, MenuDecY},
	{MenuInitX, MenuInitY}, {MenuPWMX, MenuPWMY},
	{MenuFileX, MenuFileY}, {MenuHandpadX, MenuHandpadY}, {MenuButtonsX, MenuButtonsY},
	{MenuObjectX, MenuObjectY}, {MenuTrackX, MenuTrackY},
	{MenuSidTX, MenuSidTY}, {MenuPECX, MenuPECY},
	{MenuDateTimeX, MenuDateTimeY}, {MenuMsX, MenuMsY}, {MenuGuideX, MenuGuideY},
	{MenuLX200X, MenuLX200Y},
	{MenuSlewX, MenuSlewY}, {MenuSlewX+6, MenuSlewY}, {MenuSlewX+17, MenuSlewY},
	{MenuMsParmsX, MenuMsParmsY}, {MenuMsParmsX+2, MenuMsParmsY}
};

void InitKBEvent( void);
void CloseKBEvent( void);
void PrelimMsg( void);
void ClearMenuArea( void);
void ClearMenuSubArea( void);
void WriteMainMenu( void);
void WriteMenuCats( void);
void SelectTextAttrMenuCat( const int MenuCat);
void DefaultTextAttrMenuCat( const int MenuCat);
void ChangeTextAttrMenuCat( const int MenuChoice);
void WriteSubMenus( void);
void WriteSubMenuDesc( const int MenuCat, const int SubMenu);
void SelectTextAttrMenuSub( const int MenuSub);
void MenuTextAttrMenuSub( const int MenuSub);
void ChangeTextAttrMenuSub( const int MenuChoice);
void UpdateMenuCatSub( void);
void WriteMsParmsMenu( void);
void SaveEncoderOffset( void);
void LoadEncoderOffset( void);
void SetEncoderAZandEncoderOffset( void);
Flag ConfirmHsMove( void);
void KBEventMoveHs( void);
void DrawBorder( const struct Frame P);
void KBEventInitMatrix( const int Init);
void DisplayInitStatusOnScreen( void);
void DisplayInit( void);
void WriteWindow( struct Frame P);
void RemoveWindow( struct Frame P);
void PressKeyToContMsg( const char* Msg);
void WriteHandpadStatus( void);
void WriteTrackStatus( void);
void WritePECStatus( void);
void WriteMsArcsecSec( void);
void WriteGuideArcsecSec( void);
Flag InputAltaz( struct Position* P, int X, int Y);
Flag InputEquat( struct Position* P, int X, int Y);
void GetFileName( char* Name);
Flag SelectDataFilename( char* Name);
void MenuCoordFile( void);
void ReadRa( FILE* File, struct HMSH* V);
void ReadDec( FILE* File, struct DMS* V);
Flag FillObjects( struct Object O[], int* Count);
void AddScroll( struct SCROLL* Scr);
void FreeAllScroll( void);
struct SCROLL* InitScroll( void);
void MenuScrollFile( void);
void EnterInit( const int InitToDo);
void DisplayDrift( void);
void DisplayIn( char* FileName, char* ObjectName);
void DisplayEncodersNotAvailable( void);
void DoAProcessTimedToBiosClockTick( void);
void CalcEncoderAZ( void);
void ProcessEncoders( void);
Flag WriteGuideStartup( void);
void ReadGuideStartup( void);
void GetCurDir( void);
void OptimizeInit( void);
void SortPosition( struct Position* A, struct Position* B, const double Diff);
void BubbleSortPositions( struct Position* Temp);
void DisplayHotkeys( void);
Flag EnterHsParms( void);
void DisplayMsParmsTitles( void);
void DisplayMsParmsValues( void);
void ProcessMenuSelection( void);
void SetNewMenuCatSub( void);
void GetNewMenuCatSubFromResponse( void);
void GetGrandTourRec( void);
void ProcessGrandTour( void);
void Move_Update_Handpad_Subr( void);
void ProcessAltOffset( void);
void ProcessSav1( void);
void ProcessRes1( void);
void ProcessSav2( void);
void ProcessRes2( void);
void KillInits( void);
void ProcessScroll( void);
void ProcessKBEvents( void);
void ProcessMsParmsResponse( void);
void ProcessVirtualHandpad( void);
void ProcessMenuResponse( void);

/* main */

Flag CalledByGuideFlag, KeepGoingFlag, ReadSlewFlag;

void InputEquatSlewDat( void);
void WriteEquatSlewOutFile( void);
void WriteAltazSlewOutFile( void);
void BadExit( const char* Msg);

#endif

